@@ -1,10 +1,14 @@
 import os
 import logging
 import sys
+import textwrap
 
+import cli_ui
 import yaml
 from pathlib import Path
 
+from gitlabform import EXIT_INVALID_INPUT
+
 
 class ConfigurationCore:
 
@@ -14,15 +18,15 @@ class ConfigurationCore:
     def __init__(self, config_path=None, config_string=None):
 
         if config_path and config_string:
-            logging.fatal(
+            cli_ui.fatal(
                 "Please initialize with either config_path or config_string, not both."
             )
-            sys.exit(1)
+            sys.exit(EXIT_INVALID_INPUT)
 
         try:
             if config_string:
-                logging.info("Reading config from provided string.")
-                self.config = yaml.safe_load(config_string)
+                cli_ui.debug("Reading config from provided string.")
+                self.config = yaml.safe_load(textwrap.dedent(config_string))
                 self.config_dir = "."
             else:  # maybe config_path
                 if "APP_HOME" in os.environ:
@@ -38,7 +42,7 @@ class ConfigurationCore:
                     # provided points to config.yml in the app current working dir
                     config_path = os.path.join(os.getcwd(), "config.yml")
 
-                logging.info("Reading config from file: {}".format(config_path))
+                cli_ui.debug(f"Reading config from file: {config_path}")
 
                 with open(config_path, "r") as ymlfile:
                     self.config = yaml.safe_load(ymlfile)
@@ -48,13 +52,30 @@ class ConfigurationCore:
                 self.config_dir = os.path.dirname(config_path)
 
                 if self.config.get("example_config"):
-                    logging.fatal(
+                    cli_ui.fatal(
                         "Example config detected, aborting.\n"
-                        "Haven't you forgotten to use `-c <config_file` switch?\n"
-                        "If you created your config based on the example one then please remove "
-                        "'example_config' key."
+                        "Haven't you forgotten to use `-c <config_file>` parameter?\n"
+                        "If you created your config based on the example config.yml,"
+                        " then please remove 'example_config' key."
                     )
-                    sys.exit(1)
+                    sys.exit(EXIT_INVALID_INPUT)
+
+                if self.config.get("config_version", 1) != 2:
+                    cli_ui.fatal(
+                        "This version of GitLabForm requires 'config_version: 2' entry in the config.\n"
+                        "This ensures that when the application behavior changes in a backward incompatible way,"
+                        " you won't apply unexpected configuration to your GitLab instance.\n"
+                        "Please read the upgrading guide here: https://bit.ly/3ub1g5C\n"
+                    )
+                    sys.exit(EXIT_INVALID_INPUT)
+
+                try:
+                    self.config.get("projects_and_groups")
+                except KeyNotFoundException:
+                    cli_ui.fatal(
+                        "'projects_and_groups' key in the config is required."
+                    )
+                    sys.exit(EXIT_INVALID_INPUT)
 
         except (FileNotFoundError, IOError):
             raise ConfigFileNotFoundException(config_path)
